package gssim.db.dao;

import gssim.db.find.Finder;

import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.List;


/**
 * The Interface DAOInterface.
 *
 * @param <T> the generic type
 * 
 * @author Marcin Krystek, Jarek Szymczak
 */
public interface DAOInterface <T> {

	/**
	 * Fills object from result set.
	 *
	 * @param obj the object to be filled
	 * @param result the result set
	 * @throws SQLException the SQL exception
	 */
	public void fillObjectFromResultSet(T obj, ResultSet result) throws SQLException;
	
	/**
	 * Fills statement with object.
	 *
	 * @param stmt the statement
	 * @param obj the object
	 * @param offset the offset (start parameter index in statement)
	 * @return last filled parameter index in statement
	 * @throws SQLException the SQL exception
	 */
	public int fillStatementWithObject(PreparedStatement stmt, T obj, int offset) throws SQLException;
	
	/**
	 * Insert.
	 *
	 * @param obj the object
	 * @param commit true means, that operation will be commited, false - that not
	 * @return number of rows affected by the operation
	 * @throws SQLException the SQL exception
	 */
	public int insert(T obj, boolean commit) throws SQLException;
	
	/**
	 * Insert batch.
	 *
	 * @param objList the list of objects
	 * @param commit true means, that operation will be commited, false - that not
	 * @return array with integers describing how many rows were affected by each insert operation
	 * @throws SQLException the SQL exception
	 */
	public int[] insertBatch(List<T> objList, boolean commit) throws SQLException;
	
	/**
	 * Delete.
	 *
	 * @param obj the object
	 * @param commit true means, that operation will be commited, false - that not
	 * @return true, if successful
	 * @throws SQLException the SQL exception
	 */
	public boolean delete(T obj, boolean commit) throws SQLException;
	
	/**
	 * Find.
	 *
	 * @param finder the finder
	 * @return the list of elements which were found
	 * @throws SQLException the SQL exception
	 */
	public List<T> find(Finder finder) throws SQLException;
	
	/**
	 * Select.
	 *
	 * @param obj the object (with something that determines it)
	 * @return the object
	 * @throws SQLException the SQL exception
	 */
	public T select(T obj) throws SQLException;
	
	/**
	 * Update.
	 *
	 * @param obj the object
	 * @param commit true means, that operation will be commited, false - that not
	 * @return true, if successful
	 * @throws SQLException the SQL exception
	 */
	public boolean update(T obj, boolean commit) throws SQLException;
	
	/**
	 * Commit.
	 *
	 * @throws SQLException the SQL exception
	 */
	public void commit() throws SQLException;
	
	/**
	 * Close.
	 *
	 * @throws SQLException the SQL exception
	 */
	public void close() throws SQLException;
	
}
